@@ -32,19 +32,13 @@ first_arg <- function(x) {
   purrr::compact(purrr::map(x, ~ dplyr::first(call_args(.))))
 }
 
-# regular time interval is obtained from the minimal time distance.
-# duplicated time entries result in 0L.
-# if validate = FALSE in as_tsibble, skip to check duplicated entries
-min_interval <- function(x, duplicated = TRUE) {
+# regular time interval is obtained from the greatest common divisor of positive
+# time distances.
+min_interval <- function(x) {
   if (has_length(x, 1)) { # only one time index
-    return(NA_integer_)
-  }
-  abs_diff <- abs(diff(as.numeric(x), na.rm = TRUE))
-  if (duplicated) {
-    return(minp(abs_diff)) # faster than min(abs_diff[abs_diff > 0])
-  } else {
-    min(abs_diff)
+    return(NA_real_)
   }
+  gcd_interval(x)
 }
 
 validate_vars <- function(j, x) { # j = quos/chr/dbl
@@ -63,10 +57,6 @@ surround <- function(x, bracket = "(") {
   }
 }
 
-quo_text2 <- function(x) {
-  quo_text(x, width = 500L)
-}
-
 min0 <- function(x) {
   min(x, na.rm = TRUE)
 }
@@ -74,8 +64,3 @@ min0 <- function(x) {
 max0 <- function(x) {
   max(x, na.rm = TRUE)
 }
-
-is_named <- function(quos) {
-  named_quos <- names(quos)
-  is_false(all(named_quos == ""))
-}
