@@ -3,8 +3,6 @@ library(dplyr)
 context("dplyr verbs for tsibble")
 
 test_that("group_by()", {
-  expect_error(group_by(tourism, State | Region), "Incorrect nesting")
-  expect_error(group_by(tourism, State | Region, Purpose, "Invalid tsibble"))
   grped_df <- pedestrian %>%
     group_by(Date) %>%
     group_by(Sensor, add = TRUE)
@@ -18,16 +16,24 @@ test_that("group_by()", {
     vapply(group_indices(grped_df), length, integer(1)),
     key_size(grped_df)
   )
-  expect_error(pedestrian %>% group_by(sensor = Sensor), "must not be named")
+  expect_named(
+    pedestrian %>% group_by(sensor = Sensor),
+    c(names(pedestrian), "sensor")
+  )
+  expect_equal(group_vars(pedestrian %>% group_by(sensor = Sensor)), "sensor")
+  expect_equal(
+    group_vars(pedestrian %>% group_by(yearmonth(Date))),
+    "yearmonth(Date)"
+  )
 
   grped_t <- tourism %>%
     group_by(Purpose) %>%
-    group_by(Region | State, add = TRUE)
-  expect_length(group_vars(grped_t), 2)
+    group_by(Region, State, add = TRUE)
+  expect_length(group_vars(grped_t), 3)
 })
 
 test_that("arrange.tbl_ts()", {
- expect_warning(tsbl1 <- arrange(tourism, Quarter), "not arranged by")
+ expect_warning(tsbl1 <- arrange(tourism, Quarter), "not sorted by")
  expect_equal(tsbl1, tourism)
  expect_false(is_ordered(tsbl1))
  expect_identical(key(tsbl1), key(tourism))
@@ -40,43 +46,44 @@ idx_year <- seq.int(1970, 2010, by = 10)
 dat_x <- tsibble(year = idx_year, value = rnorm(5), index = year)
 
 test_that("warnings for arrange a univariate time series", {
-  expect_warning(arrange(dat_x, value), "not arranged by `year`")
+  expect_warning(arrange(dat_x, value), "not sorted by `year`")
 })
 
 test_that("expect warnings from arrange.tbl_ts()", {
-  expect_warning(pedestrian %>% arrange(Time), "not arranged by")
-  expect_warning(pedestrian %>% arrange(Sensor, desc(Date_Time)), "not arranged by")
-  expect_warning(pedestrian %>% arrange(desc(Date_Time)), "not arranged by")
-  expect_warning(pedestrian %>% arrange(Count, Date_Time, Sensor), "not arranged by")
-  expect_warning(pedestrian %>% arrange(Sensor, Count, Date_Time), "not arranged by")
-  expect_warning(tbl <- pedestrian %>% arrange(Date_Time, Sensor), "not arranged by")
+  expect_is(pedestrian %>% arrange(desc(Sensor), Date_Time), "tbl_ts")
+  expect_warning(pedestrian %>% arrange(Time), "not sorted by")
+  expect_warning(pedestrian %>% arrange(Sensor, desc(Date_Time)), "not sorted by")
+  expect_warning(pedestrian %>% arrange(desc(Date_Time)), "not sorted by")
+  expect_warning(pedestrian %>% arrange(Count, Date_Time, Sensor), "not sorted by")
+  expect_warning(pedestrian %>% arrange(Sensor, Count, Date_Time), "not sorted by")
+  expect_warning(tbl <- pedestrian %>% arrange(Date_Time, Sensor), "not sorted by")
   expect_identical(tbl %>% arrange(Sensor, Date_Time), pedestrian)
   bm <- pedestrian %>%
     filter(Sensor == "Birrarung Marr")
-  expect_warning(bm %>% arrange(desc(Date_Time)), "not arranged by")
+  expect_warning(bm %>% arrange(desc(Date_Time)), "not sorted by")
 })
 
 test_that("arrange.grouped_ts()", {
   expect_warning(
-    tsbl2 <- tourism %>% group_by(Region | State) %>% arrange(Quarter),
-    "not arranged by"
+    tsbl2 <- tourism %>% group_by(Region, State) %>% arrange(Quarter),
+    "not sorted by"
   )
   expect_equal(tsbl2, tourism)
   expect_identical(key(tsbl2), key(tourism))
-  expect_identical(unname(group_vars(tsbl2)), "Region | State")
+  expect_identical(group_vars(tsbl2), c("Region",  "State"))
   expect_warning(
     tsbl3 <- tourism %>%
-      group_by(Region | State) %>%
+      group_by(Region, State) %>%
       arrange(Quarter, .by_group = TRUE),
-    "not arranged by"
+    "not sorted by"
   )
   expect_equal(tsbl3, tourism)
   expect_identical(key(tsbl3), key(tourism))
-  expect_identical(unname(group_vars(tsbl3)), "Region | State")
+  expect_identical(group_vars(tsbl3), c("Region",  "State"))
   tsbl4 <- tourism %>%
-    group_by(Region | State) %>%
+    group_by(Region, State) %>%
     arrange(Purpose, Quarter, .by_group = TRUE)
-  expect_identical(tsbl4, group_by(tourism, Region | State))
+  expect_equal(tsbl4, group_by(tourism, Region, State))
 })
 
 test_that("filter() and slice()", {
@@ -86,7 +93,7 @@ test_that("filter() and slice()", {
   expect_identical(index(tsbl1), index(tourism))
   expect_identical(tsbl1$Quarter, unique(tourism$Quarter))
   tsbl2 <- tourism %>%
-    group_by(!!! key(.)) %>%
+    group_by(Region, State, Purpose) %>%
     filter(Quarter < yearquarter(ymd("1999-01-01")))
   expect_identical(key(tsbl2), key(tourism))
   expect_identical(group_vars(tsbl2), key_vars(tourism))
@@ -97,7 +104,7 @@ test_that("filter() and slice()", {
     group_by(Purpose) %>%
     slice(1:3)
   expect_identical(dim(tsbl4), c(12L, ncol(tourism)))
-  expect_warning(slice(pedestrian, 3:1), "not arranged by `Sensor`, and `Date_Time`")
+  expect_warning(slice(pedestrian, 3:1), "not sorted by `Sensor`, and `Date_Time`")
   expect_error(slice(pedestrian, c(3, 3)), "Duplicated")
   expect_error(slice(pedestrian, 3, 3), "only accepts one expression.")
 })
@@ -105,7 +112,7 @@ test_that("filter() and slice()", {
 test_that("select() and rename()", {
   expect_error(select(tourism, Quarter), "Invalid tsibble")
   expect_error(select(tourism, Region), "must not be dropped")
-  expect_is(select(tourism, Region, drop = TRUE), "tbl_df")
+  expect_is(select(tourism, Region, .drop = TRUE), "tbl_df")
   expect_is(select(tourism, Quarter:Purpose), "tbl_ts")
   expect_equal(
     quo_name(index(select(tourism, Index = Quarter, Region:Purpose))),
@@ -116,7 +123,7 @@ test_that("select() and rename()", {
     "Index"
   )
   expect_equal(
-    key_vars(select(tourism, Bottom = Region, Quarter, State:Purpose))[[1]],
+    format(key(select(tourism, Bottom = Region, Quarter, State:Purpose)))[[1]],
     "Bottom | State"
   )
   expect_equal(
@@ -124,18 +131,41 @@ test_that("select() and rename()", {
     "Index"
   )
   expect_equal(
-    key_vars(rename(tourism, Bottom = Region))[[1]],
+    format(key(rename(tourism, Bottom = Region)))[[1]],
     "Bottom | State"
   )
 })
 
+test_that("select() with group_by()", {
+  # grouped by "key"
+  grped_ped <- pedestrian %>% group_by(Sensor)
+  expect_equal(groups(grped_ped), groups(grped_ped %>% select(Sensor, Date_Time)))
+  sel_ped <- grped_ped %>%
+    select(Key = Sensor, Date_Time)
+  expect_equal("Key", group_vars(sel_ped))
+  rned_ped <- grped_ped %>%
+    rename(Key = Sensor)
+  expect_equal("Key", group_vars(rned_ped))
+
+  grped_ped2 <- pedestrian %>% group_by(Date)
+  sel2 <- grped_ped2 %>%
+    select(Date2 = Date, Key = Sensor, Date_Time)
+  expect_equal("Date2", group_vars(sel2))
+  expect_equal("Key", key_vars(sel2))
+  sel3 <- grped_ped2 %>%
+    select(Date2 = Date, Sensor, Date_Time)
+  expect_equal("Date2", group_vars(sel3))
+  ren2 <- grped_ped2 %>% rename(Date2 = Date)
+  expect_equal("Date2", group_vars(ren2))
+})
+
 test_that("mutate()", {
   expect_error(mutate(tourism, Quarter = 1), "Invalid tsibble")
-  expect_is(mutate(tourism, Quarter = 1, drop = TRUE), "tbl_df")
+  expect_is(mutate(tourism, Quarter = 1, .drop = TRUE), "tbl_df")
   expect_error(mutate(tourism, Region = State), "Invalid tsibble")
   expect_identical(ncol(mutate(tourism, New = 1)), ncol(tourism) + 1L)
   tsbl <- tourism %>%
-    group_by(!!! key(.)) %>%
+    group_by(Region, State, Purpose) %>%
     mutate(New = 1:n())
   tsbl1 <- filter(tsbl, New == 1)
   tsbl2 <- slice(tsbl, 1)
@@ -150,14 +180,14 @@ test_that("summarise()", {
     summarise(AllTrips = sum(Trips))
   expect_identical(ncol(tsbl1), 2L)
   tsbl2 <- tourism %>%
-    group_by(!!! key(.)) %>%
+    group_by(Region, State, Purpose) %>%
     summarise(Trips = sum(Trips))
   expect_equal(tourism, tsbl2)
   expect_identical(key(tourism), key(tsbl2))
   expect_identical(index(tourism), index(tsbl2))
   expect_identical(is_regular(tourism), is_regular(tsbl2))
   tsbl3 <- tourism %>%
-    group_by(!!! key(.)) %>%
+    group_by(Region, State, Purpose) %>%
     summarise(Obs = n())
   expect_identical(nrow(tsbl3), nrow(tourism))
 
@@ -185,5 +215,66 @@ test_that("transmute()", {
 })
 
 test_that("distinct()", {
-  expect_error(tourism %>% distinct(Region, State, Purpose), "no support")
+  expect_equal(
+    tourism %>% distinct(Region, State, Purpose),
+    as_tibble(tourism) %>% distinct(Region, State, Purpose)
+  )
+})
+
+test_that("rename() scoped variants", {
+  all_names <- toupper(names(tsbl))
+  expect_equal(names(rename_all(tsbl, toupper)), all_names)
+  at_names <- c("qtr", "group", "A", "b", "C")
+  expect_equal(names(rename_at(tsbl, vars(a, c), toupper)), at_names)
+  if_names <- c("qtr", "group", LETTERS[1:3])
+  expect_equal(names(rename_if(tsbl, is.numeric, toupper)), if_names)
+})
+
+date_character <- function(x) {
+  is.Date(x) || is.character(x)
+}
+
+test_that("select() scoped variants", {
+  all_names <- names(tsbl)
+  expect_equal(names(select_all(tsbl)), all_names)
+  expect_equal(names(select_all(tsbl, toupper)), toupper(all_names))
+  at_names <- c("qtr", "group", "b")
+  expect_equal(names(select_at(tsbl, at_names)), at_names)
+  if_names <- c("qtr", "group")
+  expect_equal(names(select_if(tsbl, date_character)), if_names)
+})
+
+ref_tsbl <- tsbl %>%
+  mutate_if(is.numeric, function(x) x + 1)
+
+test_that("mutate() scoped variants", {
+  expect_equal(
+    tsbl %>%
+      mutate_if(is.numeric, function(x) x + 1),
+    ref_tsbl
+  )
+  expect_equal(
+    tsbl %>%
+      mutate_at(vars(a:c), function(x) x + 1),
+    ref_tsbl
+  )
+})
+
+ref_tsbl <- tsbl %>%
+  group_by(group) %>%
+  summarise_if(is.numeric, sum)
+
+test_that("summarise() scoped variants", {
+  expect_equal(
+    tsbl %>%
+      group_by(group) %>%
+      summarise_if(is.numeric, sum),
+    ref_tsbl
+  )
+  expect_equal(
+    tsbl %>%
+      group_by(group) %>%
+      summarise_at(vars(a:c), sum),
+    ref_tsbl
+  )
 })
